function [p, c,e] = fit(p, u, v, f, u0, v0, f0),

% FIT Fit a Bhatia-Wolf object (BHW) to data (u, v, f)
%
% q = FIT(p, u, v, f).  Fit the Bhatia-Wolf (BHW) object (p) to data 
% in column vectors (u, v, f).  The data should be in the same 
% coordinates as p. If p has been SET to cartesian for example, then data 
% will be expected to be cartesian also. 
% 
% The object returned will be a copy of p, except the coefficients 
% will be those of the polynomial fiited to the data. 
%
% q = FIT(p, u, v, f, u0, v0, f0). A constrained least squares fit.  In 
% this case, the surface is forced to pass through a certain point (u0, v0,
% f0).
%
% Example,
%
%   t = linspace(0, 360, 32); t = t(2:end); t = degtorad(t);
%   r = linspace(0, 1, 32); r = r(2:end);
%   [t, r] = meshgrid(t, r);
%   [x, y] = pol2cart(t, r);
%   z  = sphere(x, y, 7); % sphere with radius = 7
%
%   y0 = bhw(7); y0 = set(y0,'coord','cartesian');
%   y0 = fit(y0, x(:), y(:), z(:));
% 

if (nargin < 5),

    % unconstrained least squares
    D   = getM(p, u, v);   
   
    if (rank(D) == min(size(D))),
        c   = D\f;
    else
        rank(D)
        
        D0    = orth(D);
        c     = D0\f;
        c     = (inv(D'*D)*D'*D0)*c;
        e0    = norm(D*c-f);
        fprintf('  INVERSION ERROR = %2.6e\n',e0);    
    end 
    
else 
    
    % constrained least squares
    D0          = getM(p, u0, v0);
    D           = getM(p, u, v);
    c           = c_lsq(D, f, D0, f0);
    
end

e           = D*c - f;
p.coeff     = c;
% keyboard;

return